import math


def normalize_vector(vector):
    norm = math.sqrt(sum(i ** 2 for i in vector))
    return tuple(i / norm for i in vector)


def calculate_front_vector(point1, point2):
    return normalize_vector((point2['x'] - point1['x'], point2['y'] - point1['y'], point2['z'] - point1['z']))


def process_data(data):
    processed_data = []
    for i in range(len(data) - 1):
        roll = data[i]['roll']
        front = calculate_front_vector(data[i], data[i + 1])
        up = normalize_vector((roll['ux'], roll['uy'], roll['uz']))
        left = normalize_vector((roll['rx'], -roll['ry'], roll['rz']))
        processed_data.append({
            'pos': tuple([data[i]['x'], data[i]['y'], data[i]['z']]),
            'front': front,
            'up': up,
            'left': left,
        })

    processed_data.append({
        'pos': tuple([data[-1]['x'], data[-1]['y'], data[-1]['z']]),
        'front': processed_data[-2]['front'],
        'up': processed_data[-2]['up'],
        'left': processed_data[-2]['left'],
    })

    return processed_data



import argparse

from nl2elem_to_csv import nl2elem_reader
from nl2elem_to_csv import csv_writer
from nl2elem_to_csv import data_processor


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input', '-i', type=str, required=True)
    parser.add_argument('--output', '-o', type=str, required=False, default='track.csv')
    args = parser.parse_args()

    data = None
    with open(args.input, 'rb') as file:
        data = nl2elem_reader.read(file)

    processed_data = data_processor.process_data(data)
    print(processed_data)

    csv_writer.write_to_csv(processed_data, args.output)


if __name__ == '__main__':
    main()



import xml.etree.ElementTree


def read(filename) -> list[dict]:
    tree = xml.etree.ElementTree.parse(filename)
    root = tree.getroot().find('element')

    vertices = [vertex for vertex in root.iter('vertex')]
    rolls = [roll for roll in root.iter('roll')]

    if len(vertices) != len(rolls):
        raise ValueError('Number of vertices and rolls do not match')

    return [
        {
            'x': float(vertex.find('x').text),
            'y': float(vertex.find('y').text),
            'z': float(vertex.find('z').text),
            'strict': bool(vertex.find('strict') or 'false'),
            'roll': {
                'ux': float(roll.find('ux').text),
                'uy': float(roll.find('uy').text),
                'uz': float(roll.find('uz').text),
                'rx': float(roll.find('rx').text),
                'ry': float(roll.find('ry').text),
                'rz': float(roll.find('rz').text),
                'coord': float(roll.find('coord').text),
                'strict': bool(roll.find('strict').text),
            }
        } for vertex, roll in zip(vertices, rolls)
    ]
